package com.fjhx.file.utils;

import cn.hutool.extra.spring.SpringUtil;
import com.fjhx.file.entity.FileInfoSelectDto;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.service.FileInfoService;

import java.util.List;
import java.util.Map;

public class ObsFileUtil {

    private static final FileInfoService fileInfoService = SpringUtil.getBean(FileInfoService.class);
    public static final int defaultFileType = 0;

    /**
     * 保存文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        fileInfoService.saveFile(obsFileList, businessId, businessType);
    }

    /**
     * 保存文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId) {
        fileInfoService.saveFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 更新文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        fileInfoService.editFile(obsFileList, businessId, businessType);
    }

    /**
     * 更新文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId) {
        fileInfoService.editFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 删除文件
     *
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void removeFile(Long businessId, Integer businessType) {
        fileInfoService.removeFile(businessId, businessType);
    }

    /**
     * 交换ID
     * @param oldBusinessId   旧业务id
     * @param newBusinessId 新业务id
     */
    public static void exchangeBusinessId(Long oldBusinessId,Long newBusinessId) {
        fileInfoService.exchangeBusinessId(oldBusinessId, newBusinessId);
    }

    /**
     * 保存所有文件
     * @param obsFileList
     * @param businessId
     */
    public static void saveAllFile(List<ObsFile> obsFileList, Long businessId,Integer type) {
        fileInfoService.saveAllFile(obsFileList, businessId,type);
    }

    /**
     * 删除文件
     *
     * @param businessId 业务id
     */
    public static void removeFile(Long businessId) {
        fileInfoService.removeFile(businessId);
    }

    public static Map<Long, List<FileInfoVo>> getFileMap(List<Long> businessIdList) {
        FileInfoSelectDto dto = new FileInfoSelectDto();
        dto.setBusinessIdList(businessIdList);
        return fileInfoService.getList(dto);
    }

    public static Map<Long, List<FileInfoVo>> getFileMap(List<Long> businessIdList, Integer fileType) {
        FileInfoSelectDto dto = new FileInfoSelectDto();
        dto.setBusinessIdList(businessIdList);
        dto.setFileType(fileType);
        return fileInfoService.getList(dto);
    }

    /**
     * 判断文件是否已经绑定业务 是将文件复制一份并保存
     *
     * @param fileList   文件列表
     * @param businessId 业务id
     */
    public static void copyFileAndSave(List<ObsFile> fileList, Long businessId) {
        copyFileAndSave(fileList, businessId, defaultFileType);
    }

    /**
     * 判断文件是否已经绑定业务 是将文件复制一份并保存
     *
     * @param fileList     文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void copyFileAndSave(List<ObsFile> fileList, Long businessId, Integer businessType) {
        fileInfoService.copyFileAndSave(fileList, businessId, businessType);
    }

}
