package com.fjhx.flow.controller.flow;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.flow.entity.flow.dto.FlowExampleSelectDto;
import com.fjhx.flow.entity.flow.dto.FlowNodeDto;
import com.fjhx.flow.entity.flow.dto.SetStartDataDto;
import com.fjhx.flow.entity.flow.po.FlowInfo;
import com.fjhx.flow.entity.flow.vo.ApprovalRecordVo;
import com.fjhx.flow.entity.flow.vo.FlowExampleVo;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.ruoyi.common.annotation.TenantIgnore;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.core.domain.BaseSelectDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * <p>
 * 流程实例 前端控制器
 * </p>
 *
 * @author
 * @since 2023-03-16
 */
@DS(BaseSourceConstant.BASE)
@RestController
@RequestMapping("/flowExample")
public class FlowExampleController {

    @Autowired
    private FlowExampleService flowExampleService;

    /**
     * 获取流程类型
     */
    @PostMapping("/getFlowType")
    public List<FlowInfo> getFlowType() {
        return flowExampleService.getFlowType();
    }

    /**
     * 获取待处理流程实例
     */
    @PostMapping("getToBeProcessedPage")
    public Page<FlowExampleVo> getToBeProcessedPage(@RequestBody FlowExampleSelectDto dto) {
        return flowExampleService.getToBeProcessedPage(dto);
    }

    /**
     * 获取已发起流程实例
     */
    @PostMapping("/getHaveInitiatedPage")
    public Page<FlowExampleVo> getHaveInitiatedPage(@RequestBody FlowExampleSelectDto dto) {
        return flowExampleService.getHaveInitiatedPage(dto);
    }

    /**
     * 获取已处理流程实例
     */
    @PostMapping("/getProcessedPage")
    public Page<FlowExampleVo> getProcessedPage(@RequestBody FlowExampleSelectDto dto) {
        return flowExampleService.getProcessedPage(dto);
    }

    /**
     * 获取审批记录
     */
    @PostMapping("/getApprovalRecord")
    public ApprovalRecordVo getApprovalRecord(@RequestBody BaseSelectDto dto) {
        return flowExampleService.getApprovalRecord(dto.getId());
    }

    /**
     * 获取流程分页列表
     */
    @PostMapping("/page")
    public Page<FlowExampleVo> page(@RequestBody FlowExampleSelectDto dto) {
        return flowExampleService.getPage(dto);
    }

    /**
     * 发起流程记录
     */
    @PostMapping("/getFlowNode")
    public List<ApprovalRecordVo.Record> getFlowNode(@Validated @RequestBody FlowNodeDto dto) {
        return flowExampleService.getFlowNode(dto);
    }

    /**
     * 重新赋值开始节点数据
     */
    @TenantIgnore
    @PostMapping("setStartData")
    public void setStartData(@Validated @RequestBody SetStartDataDto dto) {
        flowExampleService.setStartData(dto);
    }

}
