package com.fjhx.flow.core;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.extern.slf4j.Slf4j;

import java.util.Date;
import java.util.Map;

@Slf4j
public abstract class FlowDelegate {

    public Map<String, Object> initTemplateMap(JSONObject currentData, JSONObject startData) {

        Map<String, Object> map = BeanUtil.beanToMap(currentData);
        map.put("startData", BeanUtil.beanToMap(startData));
        map.put("nickName", SecurityUtils.getLoginUser().getUser().getNickName());
        map.put("date", DateUtil.formatDate(new Date()));
        map.put("dateTime", DateUtil.formatDateTime(new Date()));

        return map;
    }

    /**
     * 获取流程key
     *
     * @return 流程key
     */
    public abstract String getFlowKey();

    /**
     * 开始流程实现方法
     *
     * @param flowId     流程id
     * @param submitData 流程入参
     * @return 业务id
     */
    public abstract Long start(Long flowId, JSONObject submitData);

    /**
     * 默认结束流程实现方法
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param submitData 流程入参
     */
    public abstract void end(Long flowId, Long businessId, JSONObject submitData);

    /**
     * 流程默认执行方法
     *
     * @param flowId         流程id
     * @param businessId     业务id
     * @param handleNodeType 处理节点类型
     * @param flowStatus     流程状态
     * @param submitData     流程入参
     */
    public void defaultMethod(Long flowId, Long businessId, NodeTypeEnum handleNodeType,
                              FlowStatusEnum flowStatus, JSONObject submitData) {

        // 流程驳回
        if (FlowStatusEnum.REJECT.equals(flowStatus)) {
            reject(flowId, businessId, flowStatus);
        }

        // 流程退回到发起人
        else if (FlowStatusEnum.READY_START.equals(flowStatus)) {
            returnToOriginator(flowId, businessId, flowStatus);
        }

        // 流程重新发起
        else if (NodeTypeEnum.START.equals(handleNodeType)) {
            relaunch(flowId, businessId, flowStatus, submitData);
        }

    }

    /**
     * 退回到发起人
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    public void returnToOriginator(Long flowId, Long businessId, FlowStatusEnum flowStatus) {

    }

    /**
     * 重新发起
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     * @param submitData 发起参数
     */
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {

    }

    /**
     * 驳回
     *
     * @param flowId     流程id
     * @param businessId 业务id
     * @param flowStatus 流程状态枚举
     */
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {

    }

}
