package com.fjhx.flow.core;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.fjhx.flow.entity.flow.dto.JumpDto;
import com.fjhx.flow.entity.flow.po.FlowDefinitionNode;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.HandleTypeEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.fjhx.flow.service.flow.FlowDefinitionNodeService;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.ruoyi.common.exception.ServiceException;
import lombok.Getter;
import lombok.Setter;

import java.util.List;
import java.util.Objects;

@Getter
@Setter
public class FlowJumpContext {

    public static final FlowExampleService flowExampleService = SpringUtil.getBean(FlowExampleService.class);
    public static final FlowDefinitionNodeService flowDefinitionNodeService = SpringUtil.getBean(FlowDefinitionNodeService.class);

    public FlowJumpContext(JumpDto dto) {

        jumpDto = dto;

        // 获取流程实例
        flowExample = flowExampleService.getById(dto.getFlowId());
        if (flowExample == null) {
            throw new ServiceException("没有找到流程");
        }
        if (ObjectUtil.notEqual(flowExample.getVersion(), dto.getVersion())) {
            throw new ServiceException("流程已被处理");
        }

        // 获取流程委托对象
        flowDelegate = FlowBean.getBean(flowExample.getFlowKey());

        // 获取流程节点列表
        flowDefinitionNodeList = flowDefinitionNodeService.list(
                q -> q.eq(FlowDefinitionNode::getFlowDefinitionId, flowExample.getDefinitionId()));

        // 获取当前审批节点
        FlowDefinitionNode tempCurrentNode = null;
        for (FlowDefinitionNode flowDefinitionNode : flowDefinitionNodeList) {
            if (Objects.equals(flowDefinitionNode.getId(), flowExample.getDefinitionNodeId())) {
                tempCurrentNode = flowDefinitionNode;
                break;
            }
        }
        if (tempCurrentNode == null) {
            throw new ServiceException("没有找到当前审批节点");
        }
        currentNode = tempCurrentNode;

        // 当前节点类型
        currentNodeType = NodeTypeEnum.getEnum(currentNode.getNodeType());

        // 查找跳转类型
        handleType = HandleTypeEnum.getEnum(dto.getHandleType());

        // 跳转节点用户处理id
        jumpHandleUserId = dto.getHandleUserId();

    }

    /**
     * 流程提交参数
     */
    private final JumpDto jumpDto;

    /**
     * 流程实例
     */
    private final FlowExample flowExample;

    /**
     * 流程节点列表
     */
    private final List<FlowDefinitionNode> flowDefinitionNodeList;

    /**
     * 当前审批节点
     */
    private final FlowDefinitionNode currentNode;

    /**
     * 当前审批节点类型
     */
    private final NodeTypeEnum currentNodeType;

    /**
     * 跳转类型
     */
    private final HandleTypeEnum handleType;

    /**
     * 流程委托对象
     */
    private final FlowDelegate flowDelegate;

    /**
     * 跳转节点
     */
    private FlowDefinitionNode jumpNode;

    /**
     * 跳转节点处理用户id
     */
    private Long jumpHandleUserId;

    /**
     * 流程当前状态
     */
    private FlowStatusEnum flowStatus;

}
