package com.fjhx.flow.core;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.HandleTypeEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import lombok.Setter;
import org.springframework.core.NamedThreadLocal;

import java.util.Map;

@Setter
public class FlowThreadLocalUtil {

    private static final ThreadLocal<FlowThreadLocalUtil> FLOW_HOLDER = new NamedThreadLocal<>("hx-flow");

    /**
     * 流程id
     */
    private Long flowId;

    /**
     * 业务id
     */
    private Long businessId;

    /**
     * 当前节点提交参数
     */
    private JSONObject currentData;

    /**
     * 发起流程提交参数
     */
    private JSONObject startData;

    /**
     * 模板配置参数
     */
    private Map<String, Object> templateData;

    /**
     * 流程处理类型
     */
    private HandleTypeEnum handleTypeEnum;

    /**
     * 流程状态
     */
    private FlowStatusEnum flowStatusEnum;

    /**
     * 下个节点审批用户id
     */
    private Long nextHandleUserId;

    /**
     * 当前节点状态
     */
    private NodeTypeEnum currentNodeTypeEnum;


    public static Long getFlowId() {
        return FLOW_HOLDER.get().flowId;
    }

    public static Long getBusinessId() {
        return FLOW_HOLDER.get().businessId;
    }

    public static JSONObject getCurrentData() {
        return FLOW_HOLDER.get().currentData;
    }

    public static JSONObject getStartData() {
        return FLOW_HOLDER.get().startData;
    }

    public static Map<String, Object> getTemplateData() {
        return FLOW_HOLDER.get().templateData;
    }

    public static HandleTypeEnum getHandleTypeEnum() {
        return FLOW_HOLDER.get().handleTypeEnum;
    }

    public static FlowStatusEnum getFlowStatusEnum() {
        return FLOW_HOLDER.get().flowStatusEnum;
    }

    public static Long getNextHandleUserId() {
        return FLOW_HOLDER.get().nextHandleUserId;
    }

    public static NodeTypeEnum getCurrentNodeTypeEnum() {
        return FLOW_HOLDER.get().currentNodeTypeEnum;
    }

    public static FlowThreadLocalUtil create() {
        FlowThreadLocalUtil flowThreadLocalUtil = new FlowThreadLocalUtil();
        FLOW_HOLDER.set(flowThreadLocalUtil);
        return flowThreadLocalUtil;
    }

    public static void remove() {
        FLOW_HOLDER.remove();
    }

}
