package com.fjhx.socket.core;

import com.ruoyi.common.exception.ServiceException;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.HashMap;
import java.util.Map;

@Getter
@AllArgsConstructor
public enum PushTypeEnum {

    /**
     * 系统提示
     */
    SYSTEM(-1, "系统", false),
    /**
     * 推送完整未读消息列表
     */
    ANNOUNCEMENT(1, "公告", true),
    /**
     * 推送未读数量
     */
    MESSAGE(2, "消息", true),
    /**
     * 推送消息对象
     */
    PUSH_NOTIFICATION(3, "闪动消息", false);

    /**
     * 消息类型
     */
    private final int type;
    /**
     * 消息说明
     */
    private final String remark;
    /**
     * 是否保存到数据库
     */
    private final boolean save;

    private static final Map<Integer, PushTypeEnum> map = new HashMap<>();

    static {
        for (PushTypeEnum pushTypeEnum : PushTypeEnum.values()) {
            map.put(pushTypeEnum.getType(), pushTypeEnum);
        }
    }

    public static PushTypeEnum getEnum(Integer key) {
        PushTypeEnum pushTypeEnum = map.get(key);
        if (pushTypeEnum == null) {
            throw new ServiceException("未知推送类型");
        }
        return pushTypeEnum;
    }

}
