package com.fjhx.socket.core;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.socket.core.event.WebSocketOnOpenEvent;
import com.fjhx.socket.entity.push.dto.PushAnnouncementDto;
import com.fjhx.socket.entity.push.po.PushInfo;
import com.fjhx.socket.service.push.PushAnnouncementService;
import com.fjhx.socket.service.push.PushInfoService;
import com.fjhx.socket.service.push.impl.PushAnnouncementServiceImpl;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.framework.mybatis.holder.TenantHolder;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Component
public class WebSocketListener {

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private PushInfoService pushInfoService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private PushAnnouncementService pushAnnouncementService;

    /**
     * 用户连接上webSocket
     */
    @EventListener
    public void open(WebSocketOnOpenEvent event) {
        WebSocketServer webSocketServer = event.getWebSocketServer();

        PushParam pushParam = new PushParam();
        pushParam.setUserId(webSocketServer.getUserId());
        webSocketServer.sendMessage(PushTypeEnum.ANNOUNCEMENT, pushParam);
        webSocketServer.sendMessage(PushTypeEnum.MESSAGE, pushParam);
    }

    /**
     * 每5分钟同步查看一次消息公告
     */
    @Scheduled(fixedDelay = 3 * 60 * 1000)
    private void syncOrder() {

        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);
        Boolean oldIgnore = TenantHolder.isIgnore();
        TenantHolder.setIgnore(true);
        List<SysUser> sysUserList = sysUserService.list();
        List<Long> userIdList = sysUserList.stream().map(SysUser::getUserId).collect(Collectors.toList());
        TenantHolder.setIgnore(oldIgnore);

        Set<String> redisKeySet = redisCache.scan(PushAnnouncementServiceImpl.PUSH_ANNOUNCEMENT_REDIS_KEY + "*");

        if (ObjectUtil.isEmpty(redisKeySet)) {
            return;
        }

        for (String redisKey : redisKeySet) {

            PushAnnouncementDto pushAnnouncementDto = redisCache.getCacheObject(redisKey);
            Date sendTime = pushAnnouncementDto.getSendTime();
            Date endTime = pushAnnouncementDto.getEndTime();

            if (sendTime != null && sendTime.after(new Date())) {
                continue;
            }

            List<PushInfo> pushInfoList = userIdList.stream().map(item -> {
                PushInfo pushInfo = new PushInfo();
                pushInfo.setType(PushTypeEnum.ANNOUNCEMENT.getType());
                pushInfo.setAnnouncementId(pushAnnouncementDto.getId());
                pushInfo.setTitle(pushAnnouncementDto.getTitle());
                pushInfo.setBusinessType(0);
                pushInfo.setBusinessData(pushAnnouncementDto.getContent());
                pushInfo.setPushUserId(item);
                pushInfo.setPushRead(StatusConstant.NO);
                pushInfo.setFailureTime(endTime);
                return pushInfo;
            }).collect(Collectors.toList());

            pushAnnouncementDto.setStatus(StatusConstant.YES);

            redisCache.deleteObject(redisKey);
            pushInfoService.saveBatch(pushInfoList);
            pushAnnouncementService.updateById(pushAnnouncementDto);

        }

        WebSocketServer.webSocketMap.forEach((k, v) -> {
            PushParam pushParam = new PushParam();
            if (ObjectUtil.isNotEmpty(v)) {
                for (WebSocketServer webSocketServer : v) {
                    pushParam.setUserId(webSocketServer.getUserId());
                    webSocketServer.sendMessage(PushTypeEnum.ANNOUNCEMENT, pushParam);
                }
            }
        });


        DynamicDataSourceContextHolder.poll();
    }

}
