package com.fjhx.socket.core;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.fjhx.socket.core.event.WebSocketUserOfflineEvent;
import com.fjhx.socket.entity.push.po.PushInfo;
import com.fjhx.socket.service.push.PushInfoService;
import com.ruoyi.common.constant.BaseSourceConstant;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.framework.mybatis.holder.TenantHolder;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.context.ApplicationContext;

import java.util.Date;
import java.util.List;

public class WebSocketPush {

    private static final ApplicationContext applicationContext = SpringUtil.getApplicationContext();
    private static final ISysUserService sysUserService = SpringUtil.getBean(ISysUserService.class);
    private static final PushInfoService pushInfoService = SpringUtil.getBean(PushInfoService.class);

    /**
     * 发送用户消息
     */
    public static void byUser(PushTypeEnum pushTypeEnum, Long userId, String title, Integer businessType) {
        byUser(pushTypeEnum, userId, title, businessType, StringPool.EMPTY);
    }

    /**
     * 发送用户消息
     */
    public static void byUser(PushTypeEnum pushTypeEnum, Long userId, String title, Integer businessType, Object businessData) {
        PushParam pushParam = new PushParam();
        pushParam.setPushId(IdWorker.getId());
        pushParam.setUserId(userId);
        pushParam.setType(pushTypeEnum.getType());
        pushParam.setTitle(title);
        pushParam.setBusinessType(businessType);
        pushParam.setBusinessData(businessData);
        pushParam.setNewMsg(true);
        pushParam.setCreateTime(new Date());
        byUser(pushTypeEnum, userId, pushParam);
    }

    /**
     * 发送用户消息
     */
    protected static void byUser(PushTypeEnum pushTypeEnum, Long userId, PushParam pushParam) {

        // 保存用户消息
        if (pushTypeEnum.isSave()) {
            savePushInfo(pushTypeEnum, pushParam);
        }

        List<WebSocketServer> webSocketServerList = WebSocketServer.webSocketMap.get(userId);
        if (ObjectUtil.isEmpty(webSocketServerList)) {
            applicationContext.publishEvent(new WebSocketUserOfflineEvent(pushTypeEnum, pushParam));
            return;
        }

        for (WebSocketServer webSocketServer : webSocketServerList) {
            webSocketServer.sendMessage(pushTypeEnum, pushParam);
        }

    }

    /**
     * 发送租户消息
     */
    public static void byTenant(PushTypeEnum pushTypeEnum, String title, Integer businessType) {
        byTenant(pushTypeEnum, title, businessType, StringPool.EMPTY);
    }

    /**
     * 发送租户消息
     */
    public static void byTenant(PushTypeEnum pushTypeEnum, String title, Integer businessType, Object businessData) {
        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);
        List<SysUser> sysUserList = sysUserService.list();
        for (SysUser sysUser : sysUserList) {
            byUser(pushTypeEnum, sysUser.getUserId(), title, businessType, businessData);
        }
        DynamicDataSourceContextHolder.poll();
    }

    /**
     * 发送全部消息
     */
    public static void all(PushTypeEnum pushTypeEnum, String title, Integer businessType) {
        all(pushTypeEnum, title, businessType, StringPool.EMPTY);
    }

    /**
     * 发送全部消息
     */
    public static void all(PushTypeEnum pushTypeEnum, String title, Integer businessType, Object businessData) {
        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);
        Boolean oldIgnore = TenantHolder.isIgnore();
        TenantHolder.setIgnore(true);
        List<SysUser> sysUserList = sysUserService.list();
        for (SysUser sysUser : sysUserList) {
            byUser(pushTypeEnum, sysUser.getUserId(), title, businessType, businessData);
        }
        TenantHolder.setIgnore(oldIgnore);
        DynamicDataSourceContextHolder.poll();
    }

    /**
     * 推送一次消息数量
     */
    public static void pushMessageCount(Long userId) {
        List<WebSocketServer> webSocketServers = WebSocketServer.webSocketMap.get(userId);
        for (WebSocketServer webSocketServer : webSocketServers) {
            PushParam pushParam = new PushParam();
            pushParam.setUserId(webSocketServer.getUserId());
            webSocketServer.sendMessage(PushTypeEnum.ANNOUNCEMENT, pushParam);
        }
    }

    /**
     * 保存消息
     */
    private static void savePushInfo(PushTypeEnum pushTypeEnum, PushParam pushParam) {
        DynamicDataSourceContextHolder.push(BaseSourceConstant.BASE);
        PushInfo pushInfo = new PushInfo();
        pushInfo.setId(pushParam.getPushId());
        pushInfo.setType(pushTypeEnum.getType());
        pushInfo.setAnnouncementId(pushParam.getAnnouncementId());
        pushInfo.setTitle(pushParam.getTitle());
        pushInfo.setBusinessType(pushParam.getBusinessType());
        pushInfo.setBusinessData(JSONObject.toJSONString(pushParam.getBusinessData()));
        pushInfo.setPushUserId(pushParam.getUserId());
        pushInfo.setPushRead(StatusConstant.NO);
        pushInfoService.save(pushInfo);
        DynamicDataSourceContextHolder.poll();
    }

}
