package com.fjhx.socket.service.push.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.socket.entity.push.dto.PushAnnouncementDto;
import com.fjhx.socket.entity.push.dto.PushAnnouncementSelectDto;
import com.fjhx.socket.entity.push.po.PushAnnouncement;
import com.fjhx.socket.entity.push.vo.PushAnnouncementVo;
import com.fjhx.socket.mapper.push.PushAnnouncementMapper;
import com.fjhx.socket.service.push.PushAnnouncementService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 公告 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-13
 */
@Service
public class PushAnnouncementServiceImpl extends ServiceImpl<PushAnnouncementMapper, PushAnnouncement> implements PushAnnouncementService {

    @Autowired
    private RedisCache redisCache;

    public static final String PUSH_ANNOUNCEMENT_REDIS_KEY = "pushAnnouncement:";

    @Override
    public Page<PushAnnouncementVo> getPage(PushAnnouncementSelectDto dto) {
        IWrapper<PushAnnouncement> wrapper = getWrapper();
        wrapper.orderByDesc("pa", PushAnnouncement::getId);
        wrapper.eq("pa", PushAnnouncement::getTitle, dto.getKeyword());
        Page<PushAnnouncementVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<PushAnnouncementVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        UserUtil.assignmentNickName(records, BasePo::getCreateUser, PushAnnouncementVo::setCreateUserName);
        return page;
    }

    @Override
    public PushAnnouncementVo detail(Long id) {
        PushAnnouncement PushAnnouncement = this.getById(id);
        PushAnnouncementVo result = BeanUtil.toBean(PushAnnouncement, PushAnnouncementVo.class);
        return result;
    }

    @Override
    public void add(PushAnnouncementDto pushAnnouncementDto) {
        pushAnnouncementDto.setId(IdWorker.getId());
        pushAnnouncementDto.setStatus(StatusConstant.NO);
        redisCache.setCacheObject(PUSH_ANNOUNCEMENT_REDIS_KEY + pushAnnouncementDto.getId(), pushAnnouncementDto);
        this.save(pushAnnouncementDto);
    }

    @Override
    public void edit(PushAnnouncementDto pushAnnouncementDto) {
        redisCache.setCacheObject(PUSH_ANNOUNCEMENT_REDIS_KEY + pushAnnouncementDto.getId(), pushAnnouncementDto);
        this.updateById(pushAnnouncementDto);
    }

    @Override
    public void delete(Long id) {
        redisCache.deleteObject(PUSH_ANNOUNCEMENT_REDIS_KEY + id);
        this.removeById(id);
    }

}
