package com.fjhx.socket.service.push.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.socket.core.WebSocketPush;
import com.fjhx.socket.entity.push.dto.PushInfoSelectDto;
import com.fjhx.socket.entity.push.po.PushInfo;
import com.fjhx.socket.entity.push.vo.PushInfoVo;
import com.fjhx.socket.mapper.push.PushInfoMapper;
import com.fjhx.socket.service.push.PushInfoService;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;


/**
 * <p>
 * 消息推送 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-13
 */
@Service
public class PushInfoServiceImpl extends ServiceImpl<PushInfoMapper, PushInfo> implements PushInfoService {

    @Override
    public Page<PushInfoVo> getPage(PushInfoSelectDto dto) {
        IWrapper<PushInfo> wrapper = getWrapper();
        wrapper.orderByDesc("pi", PushInfo::getId);
        wrapper.eq("pi", PushInfo::getPushRead, dto.getPushRead());
        wrapper.eq("pi", PushInfo::getPushUserId, SecurityUtils.getUserId());
        wrapper.eq("pi", PushInfo::getType, dto.getType());
        wrapper.func(ObjectUtil.isNotEmpty(dto.getWhetherFlow()), q -> {
            if (StatusConstant.YES.equals(dto.getWhetherFlow())) {
                q.eq("pi", PushInfo::getBusinessType, 0);
            } else {
                q.ne("pi", PushInfo::getBusinessType, 0);
            }
        });

        Page<PushInfoVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public PushInfoVo detail(Long id) {
        PushInfo PushInfo = this.getById(id);
        PushInfoVo result = BeanUtil.toBean(PushInfo, PushInfoVo.class);
        return result;
    }

    @Override
    public void read(List<Long> idList) {
        update(q -> q.in(BaseIdPo::getId, idList)
                .set(PushInfo::getPushRead, StatusConstant.YES)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        // 已读之后重新推送一次未读消息数量
        WebSocketPush.pushMessageCount(SecurityUtils.getUserId());
    }

}
