package com.ruoyi.framework.mybatis;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.context.annotation.Configuration;

import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Configuration
public class RemoveParam implements BeanPostProcessor {

    public static final Map<String, Boolean> updateUserMap = new ConcurrentHashMap<>();
    public static final Map<String, Boolean> updateTimeMap = new ConcurrentHashMap<>();

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {

        if (bean instanceof ServiceImpl) {
            Class<?> beanClass = bean.getClass();
            try {
                // spring代理对象类名转换成普通对象类名
                String name = beanClass.getName().split("\\$")[0];
                Class<?> aClass = Class.forName(name);
                Type genericSuperclass = aClass.getGenericSuperclass();
                if (genericSuperclass instanceof ParameterizedType) {
                    ParameterizedType parameterizedType = (ParameterizedType) genericSuperclass;
                    Type[] actualTypeArguments = parameterizedType.getActualTypeArguments();
                    Type actualTypeArgument = actualTypeArguments[1];

                    Class<?> entityClass = Class.forName(actualTypeArgument.getTypeName());

                    String tableName;
                    TableName annotation = entityClass.getAnnotation(TableName.class);
                    if (annotation != null && annotation.value() != null) {
                        tableName = annotation.value();
                    } else {
                        tableName = StrUtil.toUnderlineCase(entityClass.getSimpleName());
                    }

                    Field[] fields = ReflectUtil.getFields(entityClass);

                    // 字段
                    for (Field field : fields) {
                        TableField tableField = field.getAnnotation(TableField.class);
                        if (tableField == null) {
                            continue;
                        }

                        FieldFill fill = tableField.fill();
                        if (ObjectUtil.notEqual(fill, FieldFill.INSERT_UPDATE)) {
                            continue;
                        }

                        if (ObjectUtil.equals(tableField.value(), "update_time") || field.getName().equals("updateTime")) {
                            RemoveParam.updateTimeMap.put(tableName, true);
                            continue;
                        }

                        if (ObjectUtil.equals(tableField.value(), "update_user") || field.getName().equals("updateUser")) {
                            RemoveParam.updateUserMap.put(tableName, true);
                        }
                    }

                    RemoveParam.updateTimeMap.putIfAbsent(tableName, false);
                    RemoveParam.updateUserMap.putIfAbsent(tableName, false);
                }
            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            }
        }

        return bean;
    }

}
